import os
import string
import struct
import random
import sys
import time
import base64
import urllib
import urllib2
import urlparse

_props = {}

DEFAULT_INTEGRATION_PROPERTIES_PATH="/opt/collabnet/teamforge/runtime/conf/integration.properties"

# If kconv module exists, character code conversion is enabled.
# default: false(0)
try:
  import kconv
except ImportError:
  useKconv = False
else:
  useKconv = True

# decide japanese character code in string, and convert japanese character code to utf-8.
# if cannot decide code, kconv does not convert a character code.

def toutf8(txt, log):
    if useKconv:
        log.write("Using kconv to convert encoding")
        kc = kconv.Kconv(kconv.UTF8, kconv.AUTO)
        txt = kc.convert(txt)
        return txt
    else:
        try:
          txt = unicode(txt, 'utf-8')
        except Exception, e:
          try:
            txt = unicode(txt, 'windows-1252')
          except Exception, e:
            log.write("Using UniversalDetector to convert encoding")
            detector = UniversalDetector()
            detector.feed(txt)
            detector.close()
            if detector.result["encoding"] is not None:
                log.write("Detected encoding: " + detector.result["encoding"] + " confidence: " +  str(detector.result["confidence"]))
                try:
                    txt = unicode(txt, detector.result["encoding"])
                except Exception, e:
                    txt = unicode(txt, 'raw-unicode-escape')
                    log.write("Detected encoding seems to be wrong")
            else:
                txt = unicode(txt, 'raw-unicode-escape')
                log.write("Failed to detect encoding")
        return txt

def detect_encoding(text_block, min_confidence=1.0):
  """Return the encoding used by TEXT_BLOCK, or None if the detection
  fails or lacks some MIN_CONFIDENCE (0.0 - 1.0)."""
  
  # Does the TEXT_BLOCK start with a BOM?
  for bom, encoding in [('\xef\xbb\xbf', 'utf-8'),
                        ('\xff\xfe', 'utf-16'),
                        ('\xfe\xff', 'utf-16be'),
                        ('\xff\xfe\0\0', 'utf-32'),
                        ('\0\0\xfe\xff', 'utf-32be'),
                        ]:
    if text_block.startswith(bom):
      return encoding

  # If no recognized BOM, see if chardet can help us.
  try:
    import chardet

    # If chardet can confidently claimed a match, we'll use its
    # findings.  (And if that match is 'ascii' -- which is a subset of
    # utf-8 -- we'll just call it 'utf-8' and score a zero transform.)
    resp = chardet.detect(text_block)
    if resp.get('confidence') >= min_confidence:
      encoding = resp.get('encoding')
      if encoding is "ascii":
        encoding = "utf-8"
      return encoding
  except:
    pass

  # By default ... we have no idea.
  return None

def transcode_to_utf8(text, source_encoding, force=False):
  """Transcode TEXT from SOURCE_ENCODING to UTF-8.  If FORCE is set,
  don't worry about a strict conversion -- just be sure to return
  something."""

  if not source_encoding:
    source_encoding = 'utf-8'
  if force:
    try:
      return unicode(text, source_encoding, 'replace').encode('utf-8', 'replace')
    except:
      return text
  return unicode(text, source_encoding, 'strict').encode('utf-8', 'strict')

def load(filename=DEFAULT_INTEGRATION_PROPERTIES_PATH):
    global _props

    # When filename is the default, check for the INTEGRATION_PROPERTIES_PATH environment variable as a potential override
    if filename == DEFAULT_INTEGRATION_PROPERTIES_PATH and os.environ.has_key('INTEGRATION_PROPERTIES_PATH'):
      filename = os.environ['INTEGRATION_PROPERTIES_PATH']

    propFile = open(filename, "r")
    propLine = propFile.readline()

    while propLine != '':
        if not propLine.startswith('#'):
            propInfo = propLine.split('=', 2)
            propName = propInfo[0].strip()
            if propName != '':
                propValue = propInfo[1].strip()
                _props[propName] = propValue
        propLine = propFile.readline()

    propFile.close()
    return _props

def assertProperty(propertyMap, key):
    if not key in propertyMap:
        raise Exception(key + ' must be set in integration.properties')

def get(key, default = None):
    if not _props: load()
    return _props.get(key, default)

# get a property from a map, after determining that the key is present
def getRequired(key):
    if not _props: load()
    assertProperty(_props, key)
    return _props[key]

def getBoolean(key, default = None):
    return string.lower(get(key, default)) in ["1", "true", "on"]


def normalizeRepositoryPath(path):
    # CVS workaround
    if path.startswith(':local:'): path = path[7:]
    return os.path.abspath(path)
    
def getAPIServerBaseUrl():
    return getRequired('sfmain.soap.api_server.url')

def getSOAPServiceUrl(serviceName, soapVer = ''):
    proto = ["http", "https"] [getBoolean("sfmain.integration.listener_ssl", "false")]
    host = getRequired('sfmain.integration.listener_host')
    port = getRequired('sfmain.integration.listener_port')

    if soapVer == '' and serviceName.endswith('Listener'):
        appserverUrl = proto + "://" + host + ":" + port + "/ce-soap/services/" + serviceName + "?wsdl"
    elif soapVer != '' and int(soapVer) >= 50:
        if (serviceName == 'SourceForge'):
            serviceName = 'CollabNet'

        appserverUrl = proto + "://" + host + ":" + port + "/ce-soap" + soapVer + "/services/" + serviceName + "?wsdl"
    else:
        appserverUrl = proto + "://" + host + ":" + port + "/sf-soap" + soapVer + "/services/" + serviceName + "?wsdl"
    return appserverUrl

def getIntegrationServerUrl():
  """Creates a url to the integration server"""
  proto = ["http", "https"] [getBoolean("sfmain.integration.listener_ssl", "false")]
  host = getRequired('sfmain.integration.listener_host')
  port = getRequired('sfmain.integration.listener_port')

  return proto + "://" + host + ":" + port + "/integration"


SCM_TIMESTAMP_SALT = 0x34A49f41
SCM_KEY_SEED = "kaboomastringa"
SCM_DEFAULT_SHARED_SECRET = "xnaskdxy*B R^qbiwgd"

def getSha1Hash(string):
    py_ver = sys.version_info[:2]
    msg = None

    if py_ver[0] == 2 and py_ver[1] == 4:
        import sha

        msg = sha.new(string)
    elif py_ver[0] == 2 and py_ver[1] >= 5:
        import hashlib

        msg = hashlib.sha1()
        msg.update(string)
    else:
        # Unsupported version of Python
        raise Exception('Unsupported version of Python.  Python version must be ' \
                          'greater or equal to 2.4 or less than 3.0')

    return msg.digest()

def initTimestampSalt():
    global SCM_TIMESTAMP_SALT
    h = getSha1Hash(SCM_DEFAULT_SHARED_SECRET)
    SCM_TIMESTAMP_SALT = struct.unpack("i", h[0:4])[0]

def prepareScmRequestKey(rnd, timestamp):
    rndBytes = struct.pack("i", rnd)
    timeBytes = struct.pack("i", timestamp ^ SCM_TIMESTAMP_SALT)
    passwd = get("sfmain.integration.security.shared_secret", SCM_DEFAULT_SHARED_SECRET)
    hash = getSha1Hash(passwd + rndBytes + timeBytes + passwd + SCM_KEY_SEED)
    return rndBytes + timeBytes + hash #+ "13"

def createScmRequestKey():
    key = prepareScmRequestKey(random.randint(0, 2^32-1), int(time.time()))
    return base64.encodestring(key)

initTimestampSalt()

