from app.scripts import BaseMigrationScript

import os
import app.util

class MigrationScript(BaseMigrationScript):
    
    scriptDescription = "Create symbolic link of SVN repository base location"
    
    def migrate(self):

        scmRepoDataDir = self.cfg['SCM_REPOSITORY_DATA_DIR']

        if not os.path.isdir(scmRepoDataDir):
            cmd = "mkdir -p %s" % scmRepoDataDir
            app.util.executeCommand(cmd)

        self.createLinkToSVNPath(scmRepoDataDir)

    def createLinkToSVNPath(self, scmDataDir):

        # Softlink will be created to an new SCM data directory while
        # SVN_REPOSITORY_BASE token is referring to an old location like
        # /svnroot

        svnRepoRootPath = self.cfg['SUBVERSION_REPOSITORY_BASE']

        svnSymlinkPath = "%s/svnroot" % scmDataDir
        if os.path.exists(svnSymlinkPath):
            return 
        self.createSoftLink(svnRepoRootPath, svnSymlinkPath)

    def createSoftLink(self, originalPath, symlinkPath):

        scmUser = self.cfg["HTTPD_USER"]
        scmGroup = self.cfg["HTTPD_GROUP"]
        cmd = "chown -h %s:%s %s"  % (scmUser, scmGroup, symlinkPath)

        os.symlink(originalPath, symlinkPath)
        app.util.runSystem(cmd, self.log)

